#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/*      Written by: Christopher Capraro
        Date: 6/3/92
        Supervisor: Dr. Y. Bar-Yam

           This program demonstrates a simple example of cellular automata.
        First, a two dimensional grid is set-up.  Each element of the grid
        can exist in two states, either "on" or "off".  Then a cell from
        the top row is randomly turned "on" at a specified rate (i.e. 1 every
        10 time steps). Also, after each time step the cells that are already
        "on" will move down one row in the column they are in only if the three
        neighbor cells in the row below are "off".  The process stops when a cell
        in the first row can not move down because of the constraint mentioned
        above.

        VARIABLES:

        row     ->      the number of rows in the grid (inputted by user)

        col     ->      the number of columns in the grid (inputted by user)

        rate    ->      the number of time steps pasted before randomly
                        turning a cell in the first row "on" (inputted by user)

        count   ->      array that contains the number of cells that remain
                        "on" after each trial is completed (each element
                        in the array corresponds to a different trial)

        i,j,k,l,m,n ->      indexes for FOR loops

        radnum1-2  ->      random numbers (obtain from random number generator
                           function rand() which are seeded by the function srand())

        grid    ->      two-dimensional array that stores the states of each cell

	runs    ->      the number of trials (inputted by user)

        sum2    ->      used in intermediate step to find the sum of all of the
                        cells that remained "on" after all of the trials
                        (used to find the average number of cells "on" after
                         completion of all of the trials)

        max     ->      the maximum number of cells that remained "on" after any
                        one trial
 
	min	->	the minimum number of cells that remained "on" after any
                        one trial

	ave	->	the average number of cells that remained "on" after all of the trials
			are completed

        output	->	stores character that prints output data to screen or to a file

	temp	->	outfile pointer

	tag	->	identifies how many neighbor cells are "on" below the current cell
			being looked at   
*/

void error_exit(char *s)
 {
  printf("\nERROR: %s -bye!\n",s);
  exit(1);
 }

main()
{
 int row, col, rateC, rateH, count[100];
 register int h, i;
 int j, k, run_num, c, three_below;
 int count1, count2, center_col, flag, boundary;
 int radnum1, radnum2, timestep;
 static int grid[200][200], height[200], s_col_pos[200], col_pos[200];
 int diff, runs, sum, max, min, num;
 static int dead[10000], list[10000][3], last_pos;
 float ave, ave_height_den;
 char output, name[30], substrate;
 FILE *temp, *grap;

 grap = fopen("graph.dat","w");

 printf("\nType s if output is to go to screen or f for file:");
 scanf("%s",&output);

   if(output == 's')
     temp = stdout;
   else if(output == 'f')
    {
     printf("Enter name of file:");
     scanf("%s",&name); 
     temp = fopen(name,"w");
    }
   else
     error_exit("Enter only s or f please");

 printf("Enter the g for graphite substrate of d for diamond substrate:");
 scanf("%s",&substrate);

   if(!(substrate == 'g' || substrate == 'd'))
     error_exit("Enter only g or d please");

 printf("Enter height of grid (integer value < 300):");
 scanf("%d",&row); 

 printf("Enter width of grid (integer value < 135):");
 scanf("%d",&col);

 printf("Enter the number of time steps after which a carbon atom is deposited (integer value):");
 scanf("%d",&rateC);

 printf("Enter the number of time steps after which a hydrogen atom is deposited (integer value):");
 scanf("%d",&rateH);

 printf("Enter 1 for periodic boundary conditions and 0 for regular boundary conditions:");
 scanf("%d",&boundary);
 
   if(!(boundary == 1 || boundary == 0))
     error_exit("Enter 1 or 0 only please");
 
 printf("Enter number of trials (integer value):");
 scanf("%d",&runs);
 
 printf("\n");

/* Seed function used to seed random number function */

/* srand((unsigned)time(NULL)); */

  for(h = 0;h < row;h++)
    *(height+h) = 0;

/* FOR loops that controls the number of trials */

 for(run_num = 0;run_num < runs;run_num++)
  {
   *(count+run_num) = 0;
   last_pos = 0;
   flag = 1;
   timestep = 0;

   for(h = 0;h < (row + 2);h++)
     for(i = 0;i < (col + 2);i++)
       *(*(grid+h)+i) = 0;
 
   
  for(h = 0;h < col+2;h++)
    *(col_pos+h) = 0;
 
    if(substrate == 'g')
      for(h = 0;h < (col + 2);h++)
       {
        *(*(grid+row)+h) = 3;
        *(s_col_pos+h) = 0;
       }
    else if(substrate == 'd')
     {
      for(h = 0;h < (col + 2);h += 2)
        {
         *(*(grid+row)+h) = 3;
         *(s_col_pos+h) = 0;
        }
      for(h = 1;h < (col + 2);h += 2)
        {
         *(*(grid+row+1)+h) = 3;
         *(s_col_pos+h) = 0;
        }
     }

   count1 = rateC;
   count2 = rateH;

   do
   {
 
    timestep++;

    if(count1 == rateC)
     {
      radnum1 = rand()%col + 1;
      *(*(grid)+radnum1) = 2;
      *(*(list+last_pos)) = radnum1;
      *(*(list+last_pos)+1) = 0;
      last_pos++;
      count1 = 0;
     }

    if(count2 == rateH)
     { 
      radnum2 = rand()%col + 1;
      diff = (radnum2 - radnum1);
      if(count1 == 0 && diff >= -1 && diff <= 1)
       {
        *(*(grid)+radnum1) = 0;
        last_pos--;
       }
      else
       {
        *(*(grid)+radnum2) = -1;
        *(*(list+last_pos)) = radnum2;
        *(*(list+last_pos)+1) = 0;
        last_pos++;
       }
      count2 = 0;
     }

     count1++;
     count2++;

     num = 0;

          for(c = 0;c < last_pos;c++)
          {

           j = *(*(list+c)+1);
	   k = *(*(list+c));

             if(k == 1 && boundary == 1)
              {
               for(h = 0;h < (row+2);h++)
                  *(*(grid+h)) = *(*(grid+h)+col);
              }
             else if(k == col && boundary == 1)
              {
               for(h = 0;h < (row+2);h++)
                 *(*(grid+h)+(col+1)) = *(*(grid+h)+1);
              }

           three_below = ((*(*(grid+j+1)+k-1)) + (*(*(grid+j+1)+k)) +  (*(*(grid+j+1)+k+1)));

             if(three_below == 0)
              {
               *(*(grid+j+1)+k) = *(*(grid+j)+k);
               *(*(grid+j)+k) = 0;
               *(*(list+c)+1) = j+1;
              }
             else      
              {
                 if(*(*(grid+j)+k) == 2)
                  {
                    *(col_pos+k) = row - (j+1);

                    if(three_below > 3)
                     {
                      *(*(grid+j)+k) = 3;
                      (*(s_col_pos+k)) = j;

                      if(three_below == 4) 
                       {
                        *(*(grid+(j+1))+(k+1)) = 3;
                        *(*(grid+(j+1))+(k-1)) = 3;
                        (*(s_col_pos+(k+1))) = j+1;  
                        (*(s_col_pos+(k-1))) = j+1;
                       }
                      else if(three_below == 5)
                       {
                         if(*(*(grid+(j+1))+(k+1)) == 2)
                          {
                           *(*(grid+(j+1))+(k+1)) = 3;
                           (*(s_col_pos+(k+1))) = j+1;
                          } 
                         else
                          {
                           *(*(grid+(j+1))+(k-1)) = 3;
                           (*(s_col_pos+(k-1))) = j+1;
                          } 
                       }
                     }
                   
                     if(j == 0)
                      {
                       flag = 0;
                       break;
                      }

                   *(dead+num) = c;
                   num++;
                  }
                 else 
                  {

                   *(*(grid+j)+k) = 0;
                   *(dead+num) = c;
                   num++;

                     if(three_below == 2)
                      {
                         if(*(*(grid+j+1)+k-1) == 2)
                           center_col = k-1;
                         else if(*(*(grid+j+1)+k) == 2)
                           center_col = k;
                         else
                           center_col = k+1;                     
	
                       sum = 0;

                         for(h = j;h < (j+3);h++)
                            for(i = (center_col-1);i < (center_col+2);i++)
                                sum += *(*(grid+h)+i);

                         if(sum <= 5)
                          {
                           *(*(grid+(j+1))+center_col) = 0;
                           *(col_pos+center_col) = row - (j+2);
                          }
                      }
                    else if(three_below == 4)
                     {
                      if(*(*(grid+j)+(k-2)) == 0 && (rand()%2 + 1) == 2)
                       {
                        *(*(grid+(j+1))+(k-1)) = 0;
                        *(col_pos+(k-1)) = row - (j+2);
                       }
                      else if(*(*(grid+j)+(k+2)) == 0)
                       {
                        *(*(grid+(j+1))+(k+1)) = 0;
                        *(col_pos+(k+1)) = row - (j+2);
                       } 
                     }
                    else if(three_below == 5)
                     {
                      if(*(*(grid+(j+1))+(k+1)) == 2 && *(*(grid+j)+(k+2)) == 0)
                       {
                        *(*(grid+(j+1))+(k+1)) = 0;
                        *(col_pos+(k+1)) = row - (j+2);
                       }
                      else if(*(*(grid+j)+(k-2)) == 0)
                       {
                        *(*(grid+(j+1))+(k-1)) = 0;
                        *(col_pos+(k-1)) = row - (j+2);
                       }
                    }
                 }          
            }

             if(k == 1 && boundary == 1) 
              {
               for(h = 0;h < (row+2);h++)
                  *(*(grid+h)+col) = *(*(grid+h));
              }
             else if(k == col && boundary == 1)
              {
               for(h = 0;h < (row+2);h++)
                 *(*(grid+h)+1) = *(*(grid+h)+(col+1));
              }

         }
      
    for(h = (num-1);h > -1;h--)
     {
         for(i = *(dead+h);i <= (last_pos-1);i++)
          {
           *(*(list+i)) = *(*(list+i+1));
           *(*(list+i)+1) = *(*(list+i+1)+1);
          }
         last_pos--;
     }

   }
   while(flag);
  
   fprintf(temp,"Trial number %d:\n\n",run_num+1);

/* The next few lines of code display the final results graphically.  An 'X' represents a cell
   that is "on", an 'A' represents an "anti-on" cell, and an '.' represents a cell that is "off".
*/

   for(h = 0;h < row + 2;h++)
     {
      fprintf(temp,"%4d     ",h+1);

      for(i = 1;i < (col + 1);i++)
       {
        if(*(*(grid+h)+i) == 2)
          {
           fprintf(temp,"C");
           (*(count+run_num))++;
	   (*(height+h))++;
          }
        else if(*(*(grid+h)+i) == -1) 
          fprintf(temp,"H");
        else if(*(*(grid+h)+i) == 3)
          {
           fprintf(temp,"X");
           (*(count+run_num))++;
           (*(height+h))++;
          }
	else
	  fprintf(temp,".");
       }

      fprintf(temp,"\n");
     }
 
    fprintf(temp,"\nNumber of cells on is %d\n\n",*(count+run_num));

    fprintf(temp,"Position of highest stable carbon atom in each column\n");

    for(h = 1;h < (col + 1);h++)
      fprintf(temp,"%4d  -> %d\n",h,*(s_col_pos+h)+1);
    
    fprintf(temp,"\n");

   if(run_num == 0)
    {
     fprintf(grap,"%d\n",runs);
     fprintf(grap,"%d\n",row);
     fprintf(grap,"%d\n",col);
    }

   fprintf(grap,"\n");

   for(h = 0;h < row;h++)
     {
      for(i = 1;i < (col + 1);i++)
        fprintf(grap,"%d ",*(*(grid+h)+i));
      fprintf(grap,"\n");
     }

  }

  max = *(count);
  min = *(count);

/* FOR loop finds the maximum and the minimum number of cells that remained "on" after 
   any one trial.
*/ 

    for(h = 1;h < runs;h++)
     {
      if(*(count+h) > max)
        max  = *(count+h);
      if(*(count+h) < min)
        min = *(count+h); 
     }

  fprintf(temp,"Maximum number of cells on in any one run was %d\n",max);
  fprintf(temp,"Minimum number of cells on in any one run was %d\n",min);   

  sum = 0;

    for(h = 0;h < runs;h++)
      sum += *(count+h);

  ave = ((float)sum/(float)runs);

  fprintf(temp,"Average number of cells on after %d runs is %f\n",runs,ave); 

/* Average density was calculated by dividing the average number of cells that remained "on" 
   after a certain number of trials by the total number of cells in the grid.
*/

  fprintf(temp,"Average density of cells that are on is %f percent\n\n",(ave/(float)(row*col))*100.0);

  fprintf(temp,"timesteps = %d\n\n",timestep);

  fprintf(temp,"Average number of cells that are on in each row are:\n\n");

  for(h = 0;h < row;h++)
   {
    ave_height_den = ((float)(*(height+h))/(float)runs);
    fprintf(temp,"%4d  ->  %f\n",h+1,ave_height_den);
   }
 
  if(output == 'f')
    fclose(temp);

}
