/*
 * xdraw3.c  Combining Xlib drawing with widget.  Shows keyboard and mouse input.
 * Stop it by typing q.
 * Wm. Randolph Franklin
 * Last modified: Mon Nov  4 17:24:16 EST 1991
 * Compile with -lXaw -lXmu -lXt -lX11.
 */


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <time.h>

#include <Xm/RowColumn.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>

#define PI 3.1416

Atom DISPLAY_DATA;
Atom DISPLAY_TYPE;
typedef struct {
		int ny;
		float y[1000];
	       }	display_data;

#define NPOINTS 5
XPoint  points[NPOINTS] =
{50, 50, 650, 50, 650, 650, 50, 650, 50, 50};
XRectangle  s_carb[100000], carb[100000], hyd[100000];

Display *display;
Window  window;
GC      gc, gc_yellow_dash, gc_orange, gc_green, gc_red, gc_yellow, gc_blue;
Font    font;
int     s_carb_count = 0;
int     carb_count = 0;
int     hyd_count = 0;

void error_exit(char *s)
 {
  printf("\nERROR: %s -bye!\n",s);
  exit(1);
 }

void create_atoms(w)
    Widget w;
{
 Display * dpy = XtDisplay(w);

  DISPLAY_DATA = XInternAtom(dpy, "a_display_data", 0);

  DISPLAY_TYPE = XInternAtom(dpy, "a_display_type", 0);
}

void 
Redraw(w, event)
    Widget  w;
    XExposeEvent *event;
{				/* Called when window exposed */
    if (s_carb_count)
      XFillRectangles(display, window, gc,                      /* Redraw curve */
                      s_carb, s_carb_count);


    if (carb_count)
      XFillRectangles(display, window, gc,                   /* Redraw curve */
                      carb, carb_count);

    if (hyd_count)
      XFillRectangles(display, window, gc_red,                   /* Redraw curve */
                      hyd, hyd_count);

}

void
Buttonpress(w, event)
    Widget  w;
    XButtonEvent *event;
{
 int h, i;
 int grid[1000][1000], row, col, runs;
 int trans_col, trans_row;
 char name[30];
 FILE *ofp;
 
 printf("Enter name of file:");
 scanf("%s",&name);

 ofp = fopen(name,"r");

 fscanf(ofp,"%d",&runs); 
 fscanf(ofp,"%d",&row);
 fscanf(ofp,"%d",&col);

 for(h = 0;h < row;h++)
   for(i = 0;i < col;i++)
      fscanf(ofp,"%d",&grid[h][i]); 

 trans_row = (int)((700.0-(4*(float)row))/2.0);
 trans_col =(int)((700.0-(4*(float)col))/2.0);
 
       s_carb_count = 0;
       carb_count = 0;
       hyd_count = 0;

       for(h = 0;h < row;h++)
         {
        for(i = 0;i < col;i++)
          {
             if(grid[h][i] == 2)
              {
               carb[carb_count].x = 4*i + trans_col;
               carb[carb_count].y = 4*h + trans_row;
               carb[carb_count].width = 4;
               carb[carb_count].height = 4;
               carb_count++;
              }
             else if(grid[h][i] == -1)
              {
               hyd[hyd_count].x = 4*i + trans_col;
               hyd[hyd_count].y = 4*h + trans_row;
               hyd[hyd_count].width = 4;
               hyd[hyd_count].height = 4;
               hyd_count++;
              }
             else if(grid[h][i] == 3)
              {
               s_carb[s_carb_count].x = 4*i + trans_col;
               s_carb[s_carb_count].y = 4*h + trans_row;
               s_carb[s_carb_count].width = 4;
               s_carb[s_carb_count].height = 4;
               s_carb_count++;
              }
           }
        }

           if (s_carb_count)
                XFillRectangles(display, window, gc,                      /* Redraw curve */
                                s_carb, s_carb_count);


           if (carb_count)
                XFillRectangles(display, window, gc,                   /* Redraw curve */
                                carb, carb_count);

           if (hyd_count)
                XFillRectangles(display, window, gc_red,                   /* Redraw curve */
                                hyd, hyd_count);

}

void
Keypress(w, event)              /* Processes a keypress event */
    Widget  w;
    XKeyEvent *event;
{
 XClearArea(display,window,0,0,699,699,FALSE);
}


main(argc, argv)
    int     argc;
    char  **argv;
{
    int     screen;		/* screen is an int                 */
    unsigned long black, white;
    XEvent  event;
    Window      root;
    int     i, done;
    Colormap cmap;
    XColor  yellow, orange, green, red, blue;
    Widget  topLevel, framework, widget, command;
    Arg     arg[5];

    static XtActionsRec actions[] = {{"redraw", Redraw},
    {"keypress", Keypress},
    {"buttonpress", Buttonpress}};

    topLevel = XtInitialize(
			    argv[0],	/* application name */
			    "XDraw",	/* application class name */
			    NULL,	/* application resources (not used) */
			    0,	/* application resource count */
			    &argc,	/* command line argument count */
			    argv);	/* command-line string */

    root =  DefaultRootWindow(XtDisplay(topLevel));

  /*
   * Initialize the Atoms used for the properties.
   */
    create_atoms(topLevel);

    framework = XtCreateManagedWidget("framework",
				       xmFormWidgetClass,
				       topLevel, NULL, 0);

/*  Create the column to hold the quit button
*/
    i = 0;
    XtSetArg(arg[i], XtNbackground, 1);
    i++;
    command = XtCreateManagedWidget("command",
				    xmRowColumnWidgetClass,
				    framework, arg, i); 


    i = 0;
    XtSetArg(arg[i], XtNwidth, 700);
    i++;			/* Create widget with a certain size */
    XtSetArg(arg[i], XtNheight, 700);
    i++;
    XtSetArg(arg[i], XtNbackground, 1);
    i++;
    widget = XtCreateManagedWidget("widget", widgetClass, framework,
				   arg, i);

    XtOverrideTranslations(widget,
			   XtParseTranslationTable("<Expose>: redraw()\n\
                                                    <KeyPress>: keypress()\n\
                                                    <ButtonPress>: buttonpress()"));

    display = XtDisplay(widget);

    screen = XDefaultScreen(display);

    black = XBlackPixel(display, screen);	/* Don't assume black=0 and white=1 */
    white = XWhitePixel(display, screen);

    cmap = XDefaultColormap(display, screen);
    yellow.red = (yellow.green = 65535);
    yellow.blue = 0;
    orange.red = 65535;
    orange.green = 32768;
    orange.blue = 0;
    green.red = green.blue = 0;
    green.green = 65535;
    red.red = 65535;
    red.green = red.blue = 0;

    if (DefaultDepth(display, screen) == 1) {
	printf("Screen has depth one.  Lotsa luck showing colors\n");
	yellow.pixel = orange.pixel = green.pixel = white;
	red.pixel = black;

    } else {			/* These set the pixel field with the color id */
	if (XAllocColor(display, cmap, &yellow) == 0 ||
	    XAllocColor(display, cmap, &orange) == 0 ||
	    XAllocColor(display, cmap, &red) == 0 ||
            XAllocColor(display, cmap, &blue) == 0 ||
	    XAllocColor(display, cmap, &green) == 0) {
	    printf("Can't do colors\n");
	    exit(3);
	}
    }

    XtAddActions(actions, 3);
  /*
   * Request property change event for the ROOT window.
   */
    XSelectInput(XtDisplay(topLevel), root, PropertyChangeMask);

    XtRealizeWidget(topLevel);

    window = XtWindow(widget);	/* no window until realized */

    gc = XCreateGC(display, window, 0, 0);	/* an attribute bundle */
    XSetForeground(display, gc, black);	/* must be explicit */
    XSetBackground(display, gc, white);

    font = XLoadFont(display,"rom17");
    XSetFont(display,gc,font);

    gc_yellow_dash = XCreateGC(display, window, 0, 0);	/* another bundle */
    XSetForeground(display, gc_yellow_dash, yellow.pixel);	/* must be explicit */
    XSetBackground(display, gc_yellow_dash, red.pixel);

    XSetLineAttributes(display, gc_yellow_dash, 0, LineDoubleDash, CapRound, JoinBevel);

    gc_orange = XCreateGC(display, window, 0, 0);
    XSetForeground(display, gc_orange, orange.pixel);

    gc_green = XCreateGC(display, window, 0, 0);
    XSetForeground(display, gc_green, green.pixel);

    gc_red = XCreateGC(display, window, 0, 0);
    XSetForeground(display, gc_red, red.pixel);

    gc_yellow = XCreateGC(display, window, 0, 0);
    XSetForeground(display, gc_yellow, yellow.pixel);

    gc_blue = XCreateGC(display, window, 0, 0);
    XSetForeground(display, gc_blue, 11);
    
    while(TRUE)
     {
      XtNextEvent(&event); 
      XtDispatchEvent(&event);
     }
}


