#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

/*      Written by: Christopher Capraro
        Date: 6/3/92
        Supervisor: Dr. Y. Bar-Yam

           This program demonstrates a simple example of cellular automata.
        First, a two dimensional grid is set-up.  Each element of the grid
        can exist in two states, either "on" or "off".  Then a cell from
        the top row is randomly turned "on" at a specified rate (i.e. 1 every
        10 time steps). Also, after each time step the cells that are already
        "on" will move down one row in the column they are in only if the three
        neighbor cells in the row below are "off".  The process stops when a cell
        in the first row can not move down because of the constraint mentioned
        above.

        VARIABLES:

        row     ->      the number of rows in the grid (inputted by user)

        col     ->      the number of columns in the grid (inputted by user)

        rate    ->      the number of time steps pasted before randomly
                        turning a cell in the first row "on" (inputted by user)

        count   ->      array that contains the number of cells that remain
                        "on" after each trial is completed (each element
                        in the array corresponds to a different trial)

        i,j,k,l,m,n ->      indexes for FOR loops

        radnum1-2  ->      random numbers (obtain from random number generator
                           function rand() which are seeded by the function srand())

        grid    ->      two-dimensional array that stores the states of each cell

	runs    ->      the number of trials (inputted by user)

        sum2    ->      used in intermediate step to find the sum of all of the
                        cells that remained "on" after all of the trials
                        (used to find the average number of cells "on" after
                         completion of all of the trials)

        max     ->      the maximum number of cells that remained "on" after any
                        one trial
 
	min	->	the minimum number of cells that remained "on" after any
                        one trial

	ave	->	the average number of cells that remained "on" after all of the trials
			are completed

        output	->	stores character that prints output data to screen or to a file

	temp	->	outfile pointer

	tag	->	identifies how many neighbor cells are "on" below the current cell
			being looked at   
*/

void error_exit(char *s)
 {
  printf("\nERROR: %s -bye!\n",s);
  exit(1);
 }

main()
{
 int row, col, rateC, rateH, count[100];
 register int h, i;
 int j, k, run_num, c, three_below;
 int count1, count2, center_col, flag, boundary;
 int radnum1, radnum2;
 static int grid[300][300], height[300], col_pos[300][300];
 int diff, runs, sum, max, min, num, last_pos, timestep;
 static int dead[10], list[10][3], last_row[300];
 float ave, ave_height_den;
 char output, name[30], substrate;
 double data[600];
 int    nn[2], ndim, isign;
 int    i1, i2, i3, i2rev, i3rev, ip1, ip2, ip3, ifp1, ifp2;
 int    ibit, idim, k1, k2, n, nprev, nrem, ntot;
 double wtemp,wr,wpr,wpi,wi,theta;
 double tempr,tempi;
 FILE *temp, *power;

 power = fopen("power.dat","w");

 printf("\nType s if output is to go to screen or f for file:");
 scanf("%s",&output);

   if(output == 's')
     temp = stdout;
   else if(output == 'f')
    {
     printf("Enter name of file:");
     scanf("%s",&name); 
     temp = fopen(name,"w");
    }
   else
     error_exit("Enter only s or f please");

 printf("Enter the g for graphite substrate of d for diamond substrate:");
 scanf("%s",&substrate);

   if(!(substrate == 'g' || substrate == 'd'))
     error_exit("Enter only g or d please");

 printf("Enter height of grid (integer value < 300):");
 scanf("%d",&row); 

 printf("Enter width of grid (integer value < 135):");
 scanf("%d",&col);

 printf("Enter rate at which carbon atoms are deposited (integer value):");
 scanf("%d",&rateC);

 printf("Enter rate at which hydrogen atoms are (integer value):");
 scanf("%d",&rateH);

 printf("Enter 1 for periodic boundary conditions and 0 for regular boundary conditions:");
 scanf("%d",&boundary);
 
   if(!(boundary == 1 || boundary == 0))
     error_exit("Enter 1 or 0 only please");
 
 printf("Enter number of trials (integer value):");
 scanf("%d",&runs);
 
 printf("\n");

/* Seed function used to seed random number function */

 /* srand((unsigned)time(NULL)); */

  for(h = 0;h < row;h++)
    *(height+h) = 0;

/* FOR loops that controls the number of trials */

 for(run_num = 0;run_num < runs;run_num++)
  {
   *(count+run_num) = 0;
   flag = 1;
   last_pos = 0;
   timestep = 0;

   for(h = 0;h < (row + 2);h++)
     for(i = 0;i < (col + 2);i++)
       *(*(grid+h)+i) = 0;

   for(h = 0;h < (col + 2);h++)
     {
      *(last_row+h) = 0;
       for(i = 0;i < (col + 2);i++)
         *(*(col_pos+h)+i) = 0;
     }

    if(substrate == 'g')
      for(h = 0;h < (col + 2);h++)
        {
         *(*(grid+row)+h) = 3;
         *(*(col_pos+h)) = row;
         (*(last_row+h))++;
        }
    else if(substrate == 'd')
     {
      for(h = 1;h < (col + 2);h += 2)
        {
         *(*(grid+row+1)+h) = 3;
         *(*(col_pos+h)) = row+1;
         (*(last_row+h))++;
        }
      for(h = 0;h < (col + 2);h += 2)
        {
         *(*(grid+row)+h) = 3;
         *(*(col_pos+h)) = row;
         (*(last_row+h))++;
        }
     }

   count1 = rateC;
   count2 = rateH;

   do
   {
    timestep++;

    if(count1 == rateC)
     {
      radnum1 = rand()%col + 1;
      k = radnum1;
      j = *(*(col_pos+(k-1))+(*(last_row+k-1))-1);
       for(h = 0;h < 2;h++)
         if(*(*(col_pos+(k+h))+(*(last_row+k+h))-1) < j)
           j = *(*(col_pos+(k+h))+(*(last_row+k+h))-1);
      *(*(grid+j-1)+radnum1) = 2;
      *(*(col_pos+k)+(*(last_row+k))) = j-1;
      (*(last_row+k))++;
      if(j-1 == 0)
       {
        flag = 0;
        break;
       }
      *(*(list+last_pos)) = radnum1;
      *(*(list+last_pos)+1) = j-1;
      last_pos++;
      count1 = 0;
     }

    if(count2 == rateH)
     { 
      radnum2 = rand()%col + 1;
      diff = (radnum2 - radnum1);
      if(count1 == 0 && diff >= -1 && diff <= 1)
       {
        *(*(grid+j-1)+radnum1) = 0;
        (*(last_row+k))--; 
        last_pos--;
       }
      else
       {
        k = radnum2;
        j = *(*(col_pos+(k-1))+(*(last_row+k-1))-1);
         for(h = 0;h < 2;h++)
           if(*(*(col_pos+(k+h))+(*(last_row+k+h))-1) < j)
             j = *(*(col_pos+(k+h))+(*(last_row+k+h))-1);
        *(*(grid+j-1)+radnum2) = -1;
        *(*(list+last_pos)) = radnum2;
        *(*(list+last_pos)+1) = j-1;
        last_pos++;
       }
      count2 = 0;
     }

     count1++;
     count2++;

     num = 0;

          for(c = 0;c < last_pos;c++)
          {

           j = *(*(list+c)+1);
	   k = *(*(list+c));

             if(k == 1 && boundary == 1)
              {
               for(h = 0;h < (row+2);h++)
                  *(*(grid+h)) = *(*(grid+h)+col);
              }
             else if(k == col && boundary == 1)
              {
               for(h = 0;h < (row+2);h++)
                 *(*(grid+h)+(col+1)) = *(*(grid+h)+1);
              }

           three_below = ((*(*(grid+j+1)+k-1)) + (*(*(grid+j+1)+k)) +  (*(*(grid+j+1)+k+1)));

                 if(*(*(grid+j)+k) == 2)
                  {
                    if(three_below > 3)
                     {
                      *(*(grid+j)+k) = 3;

                      if(three_below == 4) 
                       {
                        *(*(grid+(j+1))+(k+1)) = 3;
                        *(*(grid+(j+1))+(k-1)) = 3;
                       }
                      else if(three_below == 5)
                       {
                         if(*(*(grid+(j+1))+(k+1)) == 2)
                           *(*(grid+(j+1))+(k+1)) = 3;
                         else
                           *(*(grid+(j+1))+(k-1)) = 3;
                       }
                     }
                   
                   *(dead+num) = c;
                   num++;
                  }
                 else 
                  {

                   *(*(grid+j)+k) = 0;
                   *(dead+num) = c;
                   num++;

                     if(three_below == 2)
                      {
                         center_col = -10;
                         if(*(*(grid+j+1)+k-1) == 2 && *(*(grid+j)+k-2) == 0)
                           center_col = k-1;
                         else if(*(*(grid+j+1)+k) == 2) 
                           center_col = k;
                         else if(*(*(grid+j)+k+2) == 0)
                           center_col = k+1;                     
	
                         if(center_col != -10)
                          {
                           *(*(grid+(j+1))+center_col) = 0;
                           (*(last_row+center_col))--;
                          }
                      }
                    else if(three_below == 4)
                     {
                      if(*(*(grid+j)+(k-2)) == 0 && (rand()%2 + 1) == 2)
                       {
                        *(*(grid+(j+1))+(k-1)) = 0;
                        (*(last_row+(k-1)))--;
                       }
                      else if(*(*(grid+j)+(k+2)) == 0)
                       {
                        *(*(grid+(j+1))+(k+1)) = 0;
                        (*(last_row+(k+1)))--;
                       }
                     }
                    else if(three_below == 5)
                     {
                      if(*(*(grid+(j+1))+(k+1)) == 2 && *(*(grid+j)+(k+2)) == 0)
                       {
                        *(*(grid+(j+1))+(k+1)) = 0;
                        (*(last_row+(k+1)))--;
                       }
                      else if(*(*(grid+j)+(k-2)) == 0)
                       {
                        *(*(grid+(j+1))+(k-1)) = 0;
                        (*(last_row+(k-1)))--;
                       }
                    }
                 }          

             if(k == 1 && boundary == 1) 
              {
               for(h = 0;h < (row+2);h++)
                  *(*(grid+h)+col) = *(*(grid+h));
              }
             else if(k == col && boundary == 1)
              {
               for(h = 0;h < (row+2);h++)
                 *(*(grid+h)+1) = *(*(grid+h)+(col+1));
              }
         }
      
    for(h = (num-1);h > -1;h--)
     {
         for(i = *(dead+h);i <= (last_pos-1);i++)
          {
           *(*(list+i)) = *(*(list+i+1));
           *(*(list+i)+1) = *(*(list+i+1)+1);
          }
         last_pos--;
     }

   if(timestep == 500000)
    {
     i = 1; 
     for(h = 1;h < (col+1);h++)
       {
        data[i] = row-1-(*(*(col_pos+h)+(*(last_row+h))));
        data[i+1] = 0;
        i += 2;
       }
     ndim = 1;
     nn[1] = 2*col;
     isign = 1; 
     ntot = 1;
	for ( idim=1; idim<=ndim; idim++ )
	   ntot *= nn[idim];
  	nprev = 1;
	  for (idim=ndim;idim>=1;idim--) {
    n=nn[idim];
    nrem=ntot/(n*nprev);
    ip1=nprev << 1;
    ip2=ip1 * n;
    ip3 = ip2* nrem;
    i2rev=1;
    for (i2=1;i2<=ip2;i2+=ip1) {
      if ( i2 < i2rev ) {
        for (i1 = i2; i1 <= i2+ip1-2; i1+=2 ) {
          for ( i3 = i1; i3 <= ip3; i3+=ip2) {
            i3rev = i2rev+ i3 - i2;
            /* swap complex number */
            tempr=data[i3]; data[i3] = data[i3rev]; data[i3rev]=tempr;
            tempr=data[i3+1]; data[i3+1] = data[i3rev+1]; data[i3rev+1]=tempr;
          }
        }
      }
      ibit= ip2 >> 1;
      while ( ibit >= ip1 && i2rev > ibit ) {
        i2rev -= ibit;
        ibit >>= 1;
      }
      i2rev += ibit;
    }
    ifp1 = ip1;
    while (ifp1 < ip2 ) {
      ifp2 = ifp1 << 1;
      theta = isign * 6.28318530717959 / (ifp2/ip1);
      wtemp = sin ( 0.5 * theta);
      wpr   = -2.0* wtemp*wtemp;
      wpi   = sin ( theta );
      wr = 1.0;
      wi = 0.0;
      for ( i3=1; i3<=ifp1;i3+=ip1 ) {
        for ( i1=i3; i1<=i3+ip1-2;i1 +=2) {
          for (i2=i1;i2<=ip3;i2+=ifp2 ) {
            k1=i2;
            k2=k1+ifp1;
            tempr=wr*data[k2]-wi*data[k2+1];
            tempi=wr*data[k2+1] + wi*data[k2];
            data[k2] = data[k1] - tempr;
            data[k2+1] = data[k1+1] - tempi;
            data[k1] += tempr;
            data[k1+1] += tempi;
          }
        }
        wr=(wtemp=wr)*wpr-wi*wpi+wr;
        wi=wi*wpr+wtemp*wpi+wi;
      }
      ifp1=ifp2;
    }
    nprev *= n;
  }
  for(h = 1;h < (2*col)+1;h+=2)
  fprintf(power,"%6f\n",sqrt((data[h]*data[h])+(data[h+1]*data[h+1])));
  fprintf(power,"\n");
}

   }
   while(flag);
  
   if(run_num == 0)
    { 
     fprintf(temp,"%d\n",runs);
     fprintf(temp,"%d\n",row);
     fprintf(temp,"%d\n",col);
    }

   fprintf(temp,"\n");

   for(h = 0;h < row;h++)
     {
      for(i = 1;i < (col + 1);i++)
        fprintf(temp,"%d ",*(*(grid+h)+i));
      fprintf(temp,"\n");
     }

  }

  sum = 0;

    for(h = 0;h < runs;h++)
      sum += *(count+h);

  ave = ((float)sum/(float)runs);

  for(h = 0;h < row;h++)
   {
    ave_height_den = ((float)(*(height+h))/(float)runs);
   }

  if(output == 'f')
    fclose(temp);
}
